@@ -1,5 +1,5 @@
 /* property.c --- Callback property handling.
- * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2004-2012 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -100,6 +100,10 @@ map (Gsasl_session * sctx, Gsasl_property prop)
       p = &sctx->scram_salted_password;
       break;
 
+    case GSASL_CB_TLS_UNIQUE:
+      p = &sctx->cb_tls_unique;
+      break;
+
     case GSASL_SAML20_IDP_IDENTIFIER:
       p = &sctx->saml20_idp_identifier;
       break;
@@ -108,6 +112,10 @@ map (Gsasl_session * sctx, Gsasl_property prop)
       p = &sctx->saml20_redirect_url;
       break;
 
+    case GSASL_OPENID20_AUTH_IDENTIFIER:
+      p = &sctx->openid20_auth_identifier;
+      break;
+
       /* If you add anything here, remember to change change
 	 gsasl_finish() in xfinish.c and Gsasl_session in
 	 internal.h.  */
@@ -197,7 +205,7 @@ gsasl_property_set_raw (Gsasl_session * sctx, Gsasl_property prop,
  *
  * This function will not invoke the application callback.
  *
- * Return value: Return property value, if known, or %NULL if no value
+ * Return value: Return property value, if known, or NULL if no value
  *   known.
  *
  * Since: 0.2.0
@@ -235,7 +243,7 @@ gsasl_property_fast (Gsasl_session * sctx, Gsasl_property prop)
  * the corresponding property.  This behaviour of this function will
  * be removed when the obsolete callback interfaces are removed.
  *
- * Return value: Return data for property, or %NULL if no value known.
+ * Return value: Return data for property, or NULL if no value known.
  *
  * Since: 0.2.0
  **/
