@@ -1,5 +1,5 @@
 /* callbacks.c --- Implementation of gsasl callbacks.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2002-2012 Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -205,8 +205,21 @@ callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
     case GSASL_SCRAM_SALT:
       break;
 
+    case GSASL_OPENID20_AUTH_IDENTIFIER:
+      {
+	char *str = readutf8line ("Enter OpenID authentication identifier "
+				  "(e.g. \"http://example.org/\"): ");
+
+	gsasl_property_set (sctx, GSASL_OPENID20_AUTH_IDENTIFIER, str);
+
+	rc = GSASL_OK;
+      }
+      break;
+
     default:
-      printf ("Mechanism requested unsupported property `%d'.\n", prop);
+      fprintf (stderr,
+	       "warning: mechanism requested unsupported property `%d'\n",
+	       prop);
       break;
     }
 
