@@ -1,5 +1,5 @@
 /* imap.c --- Implement IMAP profile of SASL login.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2002-2012 Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -110,8 +110,11 @@ imap_authenticate (const char *mech)
     {
       char *buf;
       int rc;
+      int len;
 
-      asprintf (&buf, ". AUTHENTICATE %s", mech);
+      len = asprintf (&buf, ". AUTHENTICATE %s", mech);
+      if (len < 0)
+	return 0;
       rc = writeln (buf);
       free (buf);
       if (!rc)
@@ -126,11 +129,14 @@ imap_step_send (const char *data)
 {
   char *buf;
   int rc;
+  int len;
 
   if (args_info.server_flag)
-    asprintf (&buf, "+ %s", data);
+    len = asprintf (&buf, "+ %s", data);
   else
-    asprintf (&buf, "%s", data);
+    len = asprintf (&buf, "%s", data);
+  if (len < 0)
+    return 0;
   rc = writeln (buf);
   free (buf);
   if (!rc)
@@ -173,8 +179,17 @@ imap_auth_finish (void)
 {
   char *in;
 
-  if (!readln (&in))
-    return 0;
+  for (;;)
+    {
+      if (!readln (&in))
+	return 0;
+
+      /* skip untagged responses which can be returned by the server after
+	 authentication (e.g. dovecot returns new '* CAPABILITY' information
+	 before the final '. OK'). */
+      if (in[0] != '*')
+	break;
+    }
 
   return 1;
 }
