@@ -1,5 +1,5 @@
 /* smtp.c --- Implement SMTP profile of SASL login.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2002-2012 Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -114,8 +114,11 @@ smtp_authenticate (const char *mech)
     {
       char *buf;
       int rc;
+      int len;
 
-      asprintf (&buf, "AUTH %s", mech);
+      len = asprintf (&buf, "AUTH %s", mech);
+      if (len < 0)
+	return 0;
       rc = writeln (buf);
       free (buf);
       if (!rc)
@@ -130,11 +133,14 @@ smtp_step_send (const char *data)
 {
   char *buf;
   int rc;
+  int len;
 
   if (args_info.server_flag)
-    asprintf (&buf, "334 %s", data);
+    len = asprintf (&buf, "334 %s", data);
   else
-    asprintf (&buf, "%s", data);
+    len = asprintf (&buf, "%s", data);
+  if (len < 0)
+    return 0;
   rc = writeln (buf);
   free (buf);
   if (!rc)
