@@ -1,5 +1,5 @@
 /* digest-md5.c --- Test the DIGEST-MD5 mechanism.
- * Copyright (C) 2002, 2003, 2004, 2005, 2007, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2002-2012 Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -182,7 +182,7 @@ doit (void)
 	}
 
       if (debug)
-	printf ("S: %.*s\n", s1len, s1);
+	printf ("S: %.*s\n", (int) s1len, s1);
 
       /* Client respond... */
 
@@ -196,7 +196,7 @@ doit (void)
 	}
 
       if (debug)
-	printf ("C: %.*s\n", s2len, s2);
+	printf ("C: %.*s\n", (int) s2len, s2);
 
       /* Server finishes... */
 
@@ -210,7 +210,7 @@ doit (void)
 	}
 
       if (debug)
-	printf ("S: %.*s\n", s1len, s1);
+	printf ("S: %.*s\n", (int) s1len, s1);
 
       /* Client finishes... */
 
@@ -228,7 +228,7 @@ doit (void)
 	  /* Solaris x86 crashes here if s2 is NULL, even when s2len
 	     is 0. */
 	  if (s2len)
-	    printf ("C: %.*s\n", s2len, s2);
+	    printf ("C: %.*s\n", (int) s2len, s2);
 	  else
 	    printf ("C: \n");
 	}
